#![allow(dead_code)]
use std::sync::Arc;
use tokio::sync::RwLock;

pub struct BrainData {
    pub nodes: RwLock<Vec<Node>>,
    pub contracts: RwLock<Vec<Contract>>,
}

#[derive(Eq, Hash, PartialEq, Clone)]
pub struct Node {
    pub public_key: String,
    pub owner_key: String,
    pub country: String,
    pub city: String,
    pub ip: String,
    pub hostname: String,
    pub avail_mem_mb: u32,
    pub avail_vcpus: u32,
    pub avail_storage_gbs: u32,
    pub avail_ipv4: u32,
    pub avail_ipv6: u32,
    pub avail_ports: u32,
    pub max_ports_per_vm: u32,
}

impl Into<crate::grpc::brain::NodeListResp> for Node {
    fn into(self) -> crate::grpc::brain::NodeListResp {
        crate::grpc::brain::NodeListResp {
            node_pubkey: self.public_key,
            country: self.country,
            city: self.city,
            ip: self.ip,
            server_rating: 0,
            provider_rating: 0,
        }
    }
}

#[derive(Clone)]
pub struct Contract {
    pub uuid: String,
    pub hostname: String,
    pub admin_pubkey: String,
    pub node_pubkey: String,
    pub exposed_ports: Vec<u32>,
    pub public_ipv4: String,
    pub public_ipv6: String,
    pub disk_size_gb: u32,
    pub vcpus: u32,
    pub memory_mb: u32,
    pub kernel_sha: String,
    pub dtrfs_sha: String,
    pub created_at: String,
}

impl Into<crate::grpc::brain::VmContract> for Contract {
    fn into(self) -> crate::grpc::brain::VmContract {
        crate::grpc::brain::VmContract {
            uuid: self.uuid,
            hostname: self.hostname,
            admin_pubkey: self.admin_pubkey,
            node_pubkey: self.node_pubkey,
            exposed_ports: self.exposed_ports,
            public_ipv4: self.public_ipv4,
            public_ipv6: self.public_ipv6,
            disk_size_gb: self.disk_size_gb,
            vcpus: self.vcpus,
            memory_mb: self.memory_mb,
            kernel_sha: self.kernel_sha,
            dtrfs_sha: self.dtrfs_sha,
            created_at: self.created_at,
        }
    }
}

impl BrainData {
    pub fn new() -> Arc<Self> {
        Arc::new(Self {
            nodes: RwLock::new(Vec::new()),
            contracts: RwLock::new(Vec::new()),
        })
    }

    pub async fn insert_node(&self, node: Node) {
        let mut nodes = self.nodes.write().await;
        nodes.push(node);
    }

    pub async fn insert_contract(&self, contract: Contract) {
        let mut contracts = self.contracts.write().await;
        contracts.push(contract);
    }

    pub async fn find_nodes_by_pubkey(&self, public_key: &str) -> Option<Node> {
        let nodes = self.nodes.read().await;
        nodes.iter().cloned().find(|n| n.public_key == public_key)
    }

    pub async fn find_nodes_by_owner_key(&self, owner_key: &str) -> Option<Node> {
        let nodes = self.nodes.read().await;
        nodes.iter().cloned().find(|n| n.owner_key == owner_key)
    }

    pub async fn find_nodes_by_filters(
        &self,
        filters: &crate::grpc::brain::NodeFilters,
    ) -> Vec<Node> {
        let nodes = self.nodes.read().await;
        nodes
            .iter()
            .cloned()
            .filter(|n| {
                n.avail_ports >= filters.free_ports
                    && (!filters.offers_ipv4 || n.avail_ipv4 > 0)
                    && (!filters.offers_ipv6 || n.avail_ipv6 > 0)
                    && n.avail_vcpus >= filters.vcpus
                    && n.avail_mem_mb >= filters.memory_mb
                    && n.avail_storage_gbs >= filters.storage_gb
            })
            .collect()
    }

    pub async fn find_contract_by_uuid(&self, uuid: &str) -> Option<Contract> {
        let contracts = self.contracts.read().await;
        contracts.iter().cloned().find(|c| c.uuid == uuid)
    }

    pub async fn find_contracts_by_admin_pubkey(&self, admin_pubkey: &str) -> Vec<Contract> {
        let contracts = self.contracts.read().await;
        contracts
            .iter()
            .cloned()
            .filter(|c| c.admin_pubkey == admin_pubkey)
            .collect()
    }

    pub async fn find_contracts_by_node_pubkey(&self, node_pubkey: &str) -> Vec<Contract> {
        let contracts = self.contracts.read().await;
        contracts
            .iter()
            .cloned()
            .filter(|c| c.node_pubkey == node_pubkey)
            .collect()
    }
}
